package com.ruoyi.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.SnowflakeUtil;
import com.ruoyi.domain.*;
import com.ruoyi.mapper.ReviewDetailsResultMapper;
import com.ruoyi.mapper.SystemReviewTaskMapper;
import com.ruoyi.service.ReviewStandardService;
import com.ruoyi.service.SystemReviewTaskService;
import com.ruoyi.service.SystemReviewTaskUserRelationService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

/**
* @author wangfei
* @description 针对表【t_system_review_task(体系审查任务)】的数据库操作Service实现
* @createDate 2023-12-15 09:56:07
*/
@Transactional
@Service
public class SystemReviewTaskServiceImpl extends ServiceImpl<SystemReviewTaskMapper, SystemReviewTask> implements SystemReviewTaskService{

    @Autowired
    private ReviewDetailsResultMapper reviewDetailsResultMapper;

    @Autowired
    private ReviewStandardService reviewStandardService;

    @Autowired
    private SystemReviewTaskUserRelationService systemReviewTaskUserRelationService;

    @Autowired
    private SystemReviewTaskService systemReviewTaskService;

    @Autowired
    private ISysUserService sysUserService;

    @Override
    public void createTaskTest(String bizInfo, List<Long> userIds) {

        // 构建任务创建者
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser initiator = sysUserService.selectUserById(loginUser.getUserId());

        // 构建任务参与者
        Long leaderId = userIds.get(0);
        List<SysUser> sysUsers = sysUserService.selectUserListByIds(userIds);
        SysUser leader = null;
        for(SysUser sysUser : sysUsers) {
            if(sysUser.getUserId().equals(leaderId)) {
                leader = sysUser;
            }
        }

        // 构建并保存任务信息
        List<ReviewStandard> allStandardTree = reviewStandardService.findTree();
        SystemReviewTask task = new SystemReviewTask()
                .setNo(SnowflakeUtil.generate().toString())
                .setTaskInitiator(initiator.getNickName())
                .setTaskInitiatorDept(initiator.getDept().getDeptName())
                .setTaskStatus("NEW")
                .setBizInfo(bizInfo)
                .setLeaderId(leaderId)
                .setLeader(leader.getNickName())
                .setCreateTime(new Date())
                .setStandard(allStandardTree);

        systemReviewTaskService.save(task);


        // 构建并保存该任务所关联的审核组信息
        List<SystemReviewTaskUserRelation> relations = new ArrayList();

        for(SysUser sysUser : sysUsers) {
            SystemReviewTaskUserRelation relation = new SystemReviewTaskUserRelation()
                    .setTaskId(task.getId())
                    .setUserId(sysUser.getUserId())
                    .setName(sysUser.getNickName())
                    .setIsLeader(sysUser.getUserId().equals(leaderId) ? 1 : 0);

            relations.add(relation);
        }

        systemReviewTaskUserRelationService.saveBatch(relations);
    }

    @Override
    public SystemReviewTask loadTaskById(Long id) {
        SystemReviewTask task = getById(id);

        List<ReviewStandard> standard = task.getStandard();
        List<ReviewDetailsResult> results = reviewDetailsResultMapper.findByTaskNo(task.getNo());

        setDetailsResult(standard, results);

        return task;
    }

    private void setDetailsResult(List<ReviewStandard> standards, List<ReviewDetailsResult> results) {

        Map<Long, ReviewDetailsResult> resultMap = results.stream().collect(Collectors.toMap(ReviewDetailsResult::getReviewDetailsId, Function.identity(), (v1, v2) -> v1));

        for(ReviewStandard standard : standards) {
            List<ReviewKeyPoint> keyPointList = standard.getKeyPointList();

            for(ReviewKeyPoint keypoint : keyPointList) {
                List<ReviewDetails> reviewDetailsList = keypoint.getReviewDetailsList();

                for(ReviewDetails details : reviewDetailsList) {
                    details.setResult(resultMap.get(details.getId()));
                }
            }
        }
    }
}




